%% Zadoff-Chu Sequence
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%  T.F. Dyson
%%  Original Script:    08122016
%%  Free to use for non-commercial, academic or Standards Body use
%%  All rights reserved by author.
%% -----------------------------------------------------------------------
%%  Brute force optimizer for search to find "Best" Peak/Sidelobe ratio
%%  constant envelope Polyphase Traing Sequence.  Optimization is over the 
%%  entire Cross-Ambiguity function for a LEO Satellite Receiver in the
%%  international Maritime Mobile band.
%%
%%  Max Doppler + LO Uncertainty for +/-65 deg elevation equates to 
%%  ~ +/- 5200 Hz at 162 MHz so this is the minimum search Doppler.  
%%  Optimization is with respect to the linear cross-correlation/ambiguity function
%%  which is the relavant method for the orbital receiver due to the operating 
%%  mode needed in this CCI limited channel as opposed to Circular Correlation
%%  used to characterize CAZAC sequences, which assumes pre-alerted search.
%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Provided without guarantee or performance claims
%% ------------------------------------------------------------------------
clear all;

VERBOSE = 1;        %% Display the final Training Seq. Cross-Ambiguity 

p = 2;              %% Set-up the Zadoff-Chu type sequence parameters
q = 31;
ro = 1:p*q-1;              %% Follows Zadoff-Chu like conditions for phase partitions
K = 11;         
M = 32;                    %% Training sequence length in chips
L_Iterations = 1000000;    %% no. of iterations for optimization search per relatively prime pairs(ro,p*q).
C = exp(1i*pi2*(0:K-1)/K);

rng('shuffle','twister');

D = zeros(1,p*q);
BEST = 0;
BEST0 = 0;
idx = find(gcd(ro,p*q)==1); %% get the relatively Prime variants for optimizing
for j=1:length(idx)
    m = idx(j);             %% Try each phase branch
    for s = 1:L_Iterations  %% Search over optimization range of Polyphase angles
        ind = ceil(rand(1,M)*p*q);
        b = exp(1i*pi2*ind*ro(m)/(p*q));
        [r,l] = xcorr(b,b); %% Determine the maximum Peak/Sidelobe response
        r = abs(r);
        p1 = max(r([1:M-1 M+1:end]));
        PSL0 = dbp(max(r)/p1);
        BEST0 = PSL0;
        
        [drt,df,dl,ddf,l]= Ambiguity(b);  %% Now search over the Cross-Ambiguity Function
        pk = max(max(drt)); 
        DZ = drt;
        [lr,lc]=size(DZ);
        Kc1 = (lc+1)/2;
        Kc2 = (lr+1)/2;
        %% Exclude the main lobe region of Ambiguity Function
        for k1=Kc2-26:Kc2+26
            for k2 = Kc1-2:Kc1+2
                DZ(k1,k2) = -16 ;
            end
        end
        sl = max(max(DZ));      %% Find largest SL in Ambiguity Fcn outside of Main Lobe region
        PSL = pk-sl;            %% Compute Pk/SL ratio in dB
        if(PSL > BEST)          %% Test against Best so far
            BEST = PSL;         %% Save the Best til now and report progress
            fprintf('AC Peak:  %5.2f \n (Symbols)nAmb PSLR: %5.2f \n',BEST0,BEST);
            bb = b;
            DRT=drt;
            L = l;
            save ZadofChu32_Ambxy bb DRT;  %% Save the interim results
        end
    end
end

% BEST
save ZadofChu32_Ambxy bb DRT;         %% Save the final result
drt = DRT;
if(VERBOSE)
%% Display the final zero-Doppler Auto-correlation and Auto-Ambiguity
    [r,ll] = xcorr(bb,bb);
    r = abs(r);
    figure(1)
    plot(dbp(r/max(r)));
    grid on;

    figure(2)
    OSR = 6;
    mesh(L(dl)/OSR,df(ddf),DRT)
    dfmax = max(df(ddf));
    tsmax = max(L(dl)/OSR);
    axis([ -tsmax tsmax -dfmax dfmax -16 0]);
    colorbar
end

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%