function signal = modulate_packet(packet,corr_idx,modulation_idx,ramp_up_tau, ramp_down_tau, num_non_buffer_bits, bitrate,samplesPerBit)
% This function creates the modulated signal from a data packet using the
% AIS modulation scheme. Data is NRZI encoded, and GMSK modulation is used.
%
% Inputs:  packet   Column vector containing the binary encoded packet.
%                   Each element is either 0 or 1.
%          database Main simulation database.
%
% Outputs: signal   Signal, as a complex column vector.
%
% $Revision: 1.1 $
% $Author: coxb $
% $Date: 2007/02/23 18:10:46 $
% $Source: /home/cvsroot/ais_simulator3/simulator/simulator/Message\040Generation/Internal/modulate_packet.m,v $
% Copyright 2006 Analyticon Limited


gmsk_bt                = 0.4;
filter_half_length_bits= 2;


% Make NRZI encoded version of the packet. A zero in the binary data means
% a change in level should occur.
nrzi_packet = 1 - 2*mod(cumsum(not(packet)), 2);
if mod(corr_idx, 2) == 0
    nrzi_packet = -nrzi_packet;
end

% Upsample the NRZI packet
sps = samplesPerBit;
upsampled_packet = zeros(length(packet)*sps,1);
for i = 1:length(packet)
    upsampled_packet((i-1)*sps+1:i*sps) = nrzi_packet(i);
end

% Design the FIR filter (Gaussian)
sigma     = sps*sqrt(log(2))/(2*pi*gmsk_bt);
fhl_samps = filter_half_length_bits*sps;
for i = -fhl_samps:fhl_samps
    filter_coeff(i+fhl_samps+1) = exp(-i^2/(2*sigma^2));
end
filter_coeff = filter_coeff./sum(filter_coeff);

% Filter the packet
filtered_packet = filter(filter_coeff, 1, upsampled_packet);

% Integrate to give the modulation phase
mod_phase = zeros(length(filtered_packet),1);
for i=2:length(filtered_packet)
    mod_phase(i) = mod_phase(i-1) + filtered_packet(i-1)*pi/(2*sps);
end

% Account for effect of non-nominal modulation index
mod_phase = mod_phase*modulation_idx/0.5;

% Calculate the complex signal
signal = complex(cos(mod_phase), sin(mod_phase));

% Apply the ramp up amplitude profile
t = (0:length(signal)-1)' / (bitrate * samplesPerBit);
signal = signal .* (1 - exp(-t/ramp_up_tau));

% Work out where power ramp down should start. This must account for
% the number of non-buffer bits, the delay due to the filter, and the delay
% due to integration.
off_idx = num_non_buffer_bits * samplesPerBit + fhl_samps + 1;
off_time = t(off_idx);

% Apply the ramp down amplitude profile
signal(off_idx:end) = signal(off_idx:end) .* exp(-(t(off_idx:end)-off_time)/ramp_down_tau);

end
